using System.ComponentModel;

namespace AGS.Types
{
    public enum SpriteImportTilingDirection
    {
        [Description("Tile to the right, and then downwards")]
        Right,
        [Description("Tile downwards, and then to the right")]
        Down
    }
}
