//=============================================================================
//
// Adventure Game Studio (AGS)
//
// Copyright (C) 1999-2011 Chris Jones and 2011-2026 various contributors
// The full list of copyright holders can be found in the Copyright.txt
// file, which is part of this source code distribution.
//
// The AGS source code is provided under the Artistic License 2.0.
// A copy of this license can be found in the file License.txt and at
// https://opensource.org/license/artistic-2-0/
//
//=============================================================================
//
//
//
//=============================================================================
#ifndef __AGS_EE_DYNOBJ__ALLDYNAMICCLASSES_H
#define __AGS_EE_DYNOBJ__ALLDYNAMICCLASSES_H

#include "ac/dynobj/cc_agsdynamicobject.h"
#include "ac/dynobj/cc_audiochannel.h"
#include "ac/dynobj/cc_audioclip.h"
#include "ac/dynobj/cc_character.h"
#include "ac/dynobj/cc_dialog.h"
#include "ac/dynobj/cc_gui.h"
#include "ac/dynobj/cc_guiobject.h"
#include "ac/dynobj/cc_hotspot.h"
#include "ac/dynobj/cc_inventory.h"
#include "ac/dynobj/cc_object.h"
#include "ac/dynobj/cc_region.h"

#include "ac/dynobj/cc_serializer.h"

#endif // __AGS_EE_DYNOBJ__ALLDYNAMICCLASSES_H
