/*         ______   ___    ___
 *        /\  _  \ /\_ \  /\_ \
 *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___
 *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
 *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
 *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
 *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
 *                                           /\____/
 *                                           \_/__/
 *
 *      Palette type.
 *
 *      By Shawn Hargreaves.
 *
 *      See readme.txt for copyright information.
 */


#ifndef ALLEGRO_PALETTE_H
#define ALLEGRO_PALETTE_H

#ifdef __cplusplus
   extern "C" {
#endif

typedef struct RGB
{
   unsigned char r, g, b;
   unsigned char a; /* alpha (if available, otherwise acts as a filler) */
} RGB;

#define PAL_SIZE     256

typedef RGB PALETTE[PAL_SIZE];

#ifdef __cplusplus
   }
#endif

#endif          /* ifndef ALLEGRO_PALETTE_H */


