using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types
{
    public enum SpeechPortraitSide
    {
        Left = 0,
        Right = 1,
        Alternate = 2,
        BasedOnCharacterPosition = 3
    }
}
