//
//  AGSKit.h
//  AGSKit
//
//  Created by Nick Sonneveld on 25/06/2016.
//  Copyright © 2016 Adventure Game Studio. All rights reserved.
//

#import <Cocoa/Cocoa.h>

//! Project version number for AGSKit.
FOUNDATION_EXPORT double AGSKitVersionNumber;

//! Project version string for AGSKit.
FOUNDATION_EXPORT const unsigned char AGSKitVersionString[];

// In this header, you should import all the public headers of your framework using statements like #import <AGSKit/PublicHeader.h>


