using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types
{
    public enum CrossfadeSpeed
    {
        No = 0,
        SlowFade = 1,
        SlowishFade = 2,
        MediumFade = 3,
        FastFade = 4
    }
}
