using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types
{
    public enum GameTextType
    {
        Unknown,
        Script,
        DialogScript,
        Message,
        DialogOption,
        ItemDescription
    }
}
