#ifndef __AGS_CN_CORE__DEFVERSION_H
#define __AGS_CN_CORE__DEFVERSION_H

#define ACI_VERSION_STR      "3.6.2.20"
#if defined (RC_INVOKED) // for MSVC resource compiler
#define ACI_VERSION_MSRC_DEF  3,6,2,20
#endif

#define SPECIAL_VERSION ""

#define ACI_COPYRIGHT_YEARS "2011-2026"

#endif // __AGS_CN_CORE__DEFVERSION_H
