// Microsoft Visual C++ generated resource script.
//
#include "resource.h"

#define APSTUDIO_READONLY_SYMBOLS
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 2 resource.
//
#include <WinResrc.h>
#define IDC_STATIC -1
#include "../../Common/core/def_version.h"

/////////////////////////////////////////////////////////////////////////////
#undef APSTUDIO_READONLY_SYMBOLS

/////////////////////////////////////////////////////////////////////////////
// English (United Kingdom) resources

#if !defined(AFX_RESOURCE_DLL) || defined(AFX_TARG_ENG)
LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_UK
#pragma code_page(1252)

/////////////////////////////////////////////////////////////////////////////
//
// DATA
//

PIXEL_SHADER            DATA                    "tintshader.fxo"

PIXEL_SHADER_LEGACY     DATA                    "tintshaderLegacy.fxo"


/////////////////////////////////////////////////////////////////////////////
//
// Version
//

VS_VERSION_INFO VERSIONINFO
 FILEVERSION ACI_VERSION_MSRC_DEF
 PRODUCTVERSION ACI_VERSION_MSRC_DEF
 FILEFLAGSMASK 0x3fL
#ifdef _DEBUG
 FILEFLAGS 0x1L
#else
 FILEFLAGS 0x0L
#endif
 FILEOS 0x4L
 FILETYPE 0x1L
 FILESUBTYPE 0x0L
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "080904b0"
        BEGIN
            VALUE "Comments", "This game was created using AGS - http://www.adventuregamestudio.co.uk/"
            VALUE "CompanyName", "AGS Engine by Chris Jones et al.        "
            VALUE "FileDescription", "Adventure Game Studio run-time engine   "
            VALUE "FileVersion", ACI_VERSION_STR
            VALUE "InternalName", "acwin"
            VALUE "LegalCopyright", "AGS Copyright (c) 1999-2010 Chris Jones and " ACI_COPYRIGHT_YEARS " others."
            VALUE "OriginalFilename", "acwin.exe"
            VALUE "ProductName", "Made with Adventure Game Studio"
            VALUE "ProductVersion", ACI_VERSION_STR
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
        VALUE "Translation", 0x809, 1200
    END
END


#ifdef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// TEXTINCLUDE
//

1 TEXTINCLUDE 
BEGIN
    "resource.h\0"
END

2 TEXTINCLUDE 
BEGIN
    "#include <WinResrc.h>\r\n"
    "#define IDC_STATIC -1\r\n"
    "#include ""../../Common/core/def_version.h""\r\n"
    "\0"
END

3 TEXTINCLUDE 
BEGIN
    "\r\n"
    "\0"
END

#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
IDI_ICON                ICON                    "game-1.ico"


/////////////////////////////////////////////////////////////////////////////
//
// Dialog
//

IDD_SETUP DIALOGEX 0, 0, 316, 303
STYLE DS_SETFONT | DS_MODALFRAME | DS_CENTER | WS_POPUP | WS_CAPTION | WS_SYSMENU
CAPTION "AGS Game Settings"
FONT 8, "Tahoma", 400, 0, 0x0
BEGIN
    CTEXT           "Static",IDC_VERSION,7,266,302,9
    DEFPUSHBUTTON   "S&ave",IDOK,7,282,56,14
    PUSHBUTTON      "&Save and Run",IDOKRUN,67,282,56,14
    PUSHBUTTON      "Cancel",IDCANCEL,127,282,56,14
    CONTROL         "",IDC_TABPANEL,"SysTabControl32",0x0,7,7,302,253
    PUSHBUTTON      "Reset To Defaults",IDRESETTODEFAULTS,229,282,80,14
END

IDD_PAGE_BASIC DIALOGEX 0, 0, 270, 231
STYLE DS_SETFONT | WS_CHILD
FONT 8, "Tahoma", 400, 0, 0xCC
BEGIN
    GROUPBOX        "Graphics settings",IDC_GFXOPTIONS,7,7,247,178
    CONTROL         "Game resolution: XXXX x XXXX",IDC_RESOLUTION,"Static",SS_LEFTNOWORDWRAP | WS_GROUP,15,19,161,10
    CONTROL         "Driver:",IDC_STATIC,"Static",SS_LEFTNOWORDWRAP | WS_GROUP,14,34,35,10
    COMBOBOX        IDC_GFXDRIVER,50,32,136,100,CBS_DROPDOWNLIST | WS_VSCROLL | WS_TABSTOP
    CONTROL         "Display:",IDC_STATIC,"Static",SS_LEFTNOWORDWRAP | WS_GROUP,14,51,35,10
    COMBOBOX        IDC_DISPLAYINDEX,50,49,136,100,CBS_DROPDOWNLIST | WS_VSCROLL | WS_TABSTOP
    CONTROL         "&Start in a windowed mode",IDC_WINDOWED,"Button",BS_AUTOCHECKBOX | BS_LEFT | BS_VCENTER | WS_TABSTOP,14,71,172,9
    CONTROL         "Fullscreen mode:",IDC_STATIC,"Static",SS_LEFTNOWORDWRAP | WS_GROUP,14,88,55,8
    COMBOBOX        IDC_GFXMODE,75,86,166,100,CBS_DROPDOWNLIST | WS_VSCROLL | WS_TABSTOP
    CONTROL         "Fullscreen as borderless window",IDC_FULLSCREENDESKTOP,
                    "Button",BS_AUTOCHECKBOX | BS_LEFT | BS_VCENTER | WS_TABSTOP,14,105,172,9
    CONTROL         "Fullscreen scale:",IDC_STATIC,"Static",SS_LEFTNOWORDWRAP | WS_GROUP,14,124,56,10
    COMBOBOX        IDC_FSSCALING,75,122,166,100,CBS_DROPDOWNLIST | WS_VSCROLL | WS_TABSTOP
    CONTROL         "Windowed scale:",IDC_STATIC,"Static",SS_LEFTNOWORDWRAP | WS_GROUP,14,141,54,10
    COMBOBOX        IDC_WINDOWSCALING,75,139,166,100,CBS_DROPDOWNLIST | WS_VSCROLL | WS_TABSTOP
    CONTROL         "Scaling method:",IDC_STATIC,"Static",SS_LEFTNOWORDWRAP | WS_GROUP,14,158,56,10
    COMBOBOX        IDC_GFXFILTER,75,156,166,100,CBS_DROPDOWNLIST | WS_VSCROLL | WS_TABSTOP
    GROUPBOX        "Gameplay settings",IDC_STATIC,7,191,247,33
    CONTROL         "Game language:",IDC_STATIC,"Static",SS_LEFTNOWORDWRAP | WS_GROUP,15,205,56,10
    COMBOBOX        IDC_LANGUAGE,71,203,170,100,CBS_DROPDOWNLIST | WS_VSCROLL | WS_TABSTOP
END

IDD_PAGE_ADVANCED DIALOGEX 0, 0, 278, 233
STYLE DS_SETFONT | WS_CHILD
FONT 8, "Tahoma", 400, 0, 0xCC
BEGIN
    GROUPBOX        "Graphics options",IDC_STATIC,7,7,247,90
    CONTROL         "Vertical sync",IDC_VSYNC,"Button",BS_AUTOCHECKBOX | WS_TABSTOP,15,20,135,9
    CONTROL         "Smooth scaled sprites",IDC_ANTIALIAS,"Button",BS_AUTOCHECKBOX | WS_TABSTOP,15,33,138,9
    CONTROL         "Render sprites at screen resolution",IDC_RENDERATSCREENRES,
                    "Button",BS_AUTOCHECKBOX | BS_LEFT | BS_VCENTER | WS_TABSTOP,15,46,172,9
    CONTROL         "Sprite cache (RAM):",IDC_STATIC,"Static",SS_LEFTNOWORDWRAP | WS_GROUP,15,63,78,10
    COMBOBOX        IDC_SPRITECACHE,97,61,118,100,CBS_DROPDOWNLIST | WS_VSCROLL | WS_TABSTOP
    CONTROL         "Texture cache (VRAM):",IDC_STATIC,"Static",SS_LEFTNOWORDWRAP | WS_GROUP,15,80,78,10
    COMBOBOX        IDC_TEXTURECACHE,97,77,118,100,CBS_DROPDOWNLIST | WS_VSCROLL | WS_TABSTOP
    GROUPBOX        "Sound options",IDC_STATIC,7,99,247,64
    LTEXT           "Audio driver:",IDC_STATIC,15,114,46,9
    COMBOBOX        IDC_DIGISOUND,63,112,178,100,CBS_DROPDOWNLIST | WS_VSCROLL | WS_GROUP | WS_TABSTOP
    CONTROL         "Use voice pack if available",IDC_VOICEPACK,"Button",BS_AUTOCHECKBOX | WS_TABSTOP,15,130,160,9
    CONTROL         "Sound cache (RAM):",IDC_STATIC,"Static",SS_LEFTNOWORDWRAP | WS_GROUP,15,145,78,10
    COMBOBOX        IDC_SOUNDCACHE,97,144,118,100,CBS_DROPDOWNLIST | WS_VSCROLL | WS_TABSTOP
    GROUPBOX        "Mouse options",IDC_STATIC,7,166,247,60
    CONTROL         "Auto lock to window",IDC_MOUSE_AUTOLOCK,"Button",BS_AUTOCHECKBOX | WS_TABSTOP,15,181,80,10
    LTEXT           "Cursor speed: Default",IDC_MOUSESPEED_TEXT,15,195,176,8
    CONTROL         "",IDC_MOUSESPEED,"msctls_trackbar32",WS_TABSTOP,15,206,226,14
END

IDD_PAGE_PATHS DIALOGEX 0, 0, 276, 88
STYLE DS_SETFONT | WS_CHILD
FONT 8, "Tahoma", 400, 0, 0xCC
BEGIN
    CONTROL         "Custom game saves path",IDC_CUSTOMSAVEDIRCHECK,"Button",BS_AUTOCHECKBOX | WS_TABSTOP,15,7,201,10,WS_EX_TRANSPARENT
    GROUPBOX        "",IDC_STATIC,7,8,247,30
    EDITTEXT        IDC_CUSTOMSAVEDIR,15,19,210,14,ES_AUTOHSCROLL | ES_READONLY
    PUSHBUTTON      "...",IDC_CUSTOMSAVEDIRBTN,229,19,20,14
    CONTROL         "Custom game shared data path",IDC_CUSTOMAPPDATADIRCHECK,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,15,40,201,10,WS_EX_TRANSPARENT
    GROUPBOX        "",IDC_STATIC,7,41,247,30
    EDITTEXT        IDC_CUSTOMAPPDATADIR,15,52,210,14,ES_AUTOHSCROLL | ES_READONLY
    PUSHBUTTON      "...",IDC_CUSTOMAPPDATADIRBTN,229,52,20,14
END

IDD_PAGE_ACCESS DIALOGEX 0, 0, 290, 88
STYLE DS_SETFONT | WS_CHILD
FONT 8, "Tahoma", 400, 0, 0xCC
BEGIN
    GROUPBOX        "",IDC_STATIC,7,7,276,74
    CONTROL         "Enable Accessibility options",IDC_ACCESSENABLECHECK,
                    "Button",BS_AUTOCHECKBOX | WS_TABSTOP,15,7,201,10,WS_EX_TRANSPARENT
    CONTROL         "Speech skip style:",IDC_LABEL_SPEECHSKIPSTYLE,"Static",SS_LEFTNOWORDWRAP | WS_GROUP,14,25,78,10
    COMBOBOX        IDC_SPEECHSKIPSTYLE,100,22,118,100,CBS_DROPDOWNLIST | WS_VSCROLL | WS_TABSTOP
    CONTROL         "Text message skip style:",IDC_LABEL_TEXTSKIPSTYLE,
                    "Static",SS_LEFTNOWORDWRAP | WS_GROUP,14,41,78,10
    COMBOBOX        IDC_TEXTSKIPSTYLE,100,38,118,100,CBS_DROPDOWNLIST | WS_VSCROLL | WS_TABSTOP
    CONTROL         "Text reading speed:",IDC_LABEL_TEXTREADSPEED,"Static",SS_LEFTNOWORDWRAP | WS_GROUP,14,57,78,10
    CONTROL         "",IDC_TEXTREADSPEED,"msctls_trackbar32",TBS_BOTH | TBS_NOTICKS | WS_TABSTOP,100,55,117,15
    CONTROL         "Game Default",IDC_TEXTREADSPEED_TEXT,"Static",SS_LEFTNOWORDWRAP | WS_GROUP,221,57,55,10
END


/////////////////////////////////////////////////////////////////////////////
//
// DESIGNINFO
//

#ifdef APSTUDIO_INVOKED
GUIDELINES DESIGNINFO
BEGIN
    IDD_SETUP, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 309
        TOPMARGIN, 7
        BOTTOMMARGIN, 296
    END

    IDD_PAGE_BASIC, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 263
        TOPMARGIN, 7
        BOTTOMMARGIN, 191
    END

    IDD_PAGE_ADVANCED, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 271
        TOPMARGIN, 7
        BOTTOMMARGIN, 226
    END

    IDD_PAGE_PATHS, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 269
        TOPMARGIN, 7
        BOTTOMMARGIN, 81
    END

    IDD_PAGE_ACCESS, DIALOG
    BEGIN
        LEFTMARGIN, 7
        RIGHTMARGIN, 283
        TOPMARGIN, 7
        BOTTOMMARGIN, 81
    END
END
#endif    // APSTUDIO_INVOKED


/////////////////////////////////////////////////////////////////////////////
//
// AFX_DIALOG_LAYOUT
//

IDD_SETUP AFX_DIALOG_LAYOUT
BEGIN
    0
END

IDD_PAGE_BASIC AFX_DIALOG_LAYOUT
BEGIN
    0
END

IDD_PAGE_ADVANCED AFX_DIALOG_LAYOUT
BEGIN
    0
END

IDD_PAGE_PATHS AFX_DIALOG_LAYOUT
BEGIN
    0
END

IDD_PAGE_ACCESS AFX_DIALOG_LAYOUT
BEGIN
    0
END

#endif    // English (United Kingdom) resources
/////////////////////////////////////////////////////////////////////////////



#ifndef APSTUDIO_INVOKED
/////////////////////////////////////////////////////////////////////////////
//
// Generated from the TEXTINCLUDE 3 resource.
//


/////////////////////////////////////////////////////////////////////////////
#endif    // not APSTUDIO_INVOKED

