using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types
{
    public enum GUIClickAction
    {
        NoAction = 0,
        SetCursorMode = 1,
        RunScript = 2
    }
}
