using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types
{
    public interface IProjectTreeSingleClickHandler
    {
        void SingleClick(string controlID);
    }
}
