using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types.Enums
{
    public enum LookInDocumentType
    {
        CurrentDocument,
        CurrentProject,
        OpenDocuments,
        CurrentSelection
    }
}
