using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.CScript.Compiler
{
    public enum RequiredState
    {
        NotAllowed,
        Optional,
        Required
    }
}
