using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types
{
    public enum PaletteColourType
    {
        Gamewide = 0,
        //Locked = 1,
        Background = 2
    }
}
