using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types
{
    public enum TextParserWordType
    {
        Ignore = 0,
        Normal = 1,
        MatchAnyWord = 2,
        MatchRestOfInput = 3
    }
}
