using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Text;

namespace AGS.Types
{
    public class GameTemplate : BaseTemplate
    {
        public GameTemplate(string fileName, string description, Icon icon)
            : base(fileName, description, icon)
        {
        }
    }
}
