using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types.AutoComplete
{
    public class ScriptToken
    {
        public string IfDefOnly;
        public string IfNDefOnly;
		public int StartsAtCharacterIndex;
        public string Description;
    }
}
