using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types
{
    public enum RoomVolumeAdjustment
    {
        Quietest = -3,
        Quieter = -2,
        Quiet = -1,
        Normal = 0,
        Loud = 1,
        Louder = 2,
        Loudest = 3
    }
}
