using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types
{
    public class AGSSerializeClassAttribute : Attribute
    {
        public AGSSerializeClassAttribute()
            : base()
        {
        }
    }
}
