using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types
{
    public enum SpeechStyle
    {
        Lucasarts = 0,
        SierraTransparent = 1,
        SierraWithBackground = 2,
        WholeScreen = 3
    }
}
