using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types
{
    /// <summary>
    /// This needs to match the enum definition in agsdefns.sh
    /// </summary>
    public enum AudioClipFileType
    {
        OGG = 1,
        MP3 = 2,
        WAV = 3,
        VOC = 4,
        MIDI = 5,
        MOD = 6
    }
}
