//=============================================================================
//
// Adventure Game Studio (AGS)
//
// Copyright (C) 1999-2011 Chris Jones and 2011-2026 various contributors
// The full list of copyright holders can be found in the Copyright.txt
// file, which is part of this source code distribution.
//
// The AGS source code is provided under the Artistic License 2.0.
// A copy of this license can be found in the file License.txt and at
// https://opensource.org/license/artistic-2-0/
//
//=============================================================================

#if defined (OBSOLETE)
#include <stdio.h>
#include "ac/interfaceelement.h"

InterfaceElement::InterfaceElement() {
    vtextxp = 0; vtextyp = 1; snprintf(vtext, sizeof(vtext), "%s", "@SCORETEXT@$r@GAMENAME@");
    numbuttons = 0; bgcol = 8; fgcol = 15; bordercol = 0; on = 1; flags = 0;
}

#endif // OBSOLETE
