using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Editor
{
    public enum BackgroundAutoCompleteStatus
    {
        Processing,
        Finished,
        Error
    }
}
