using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.CScript.Compiler
{
    /// <summary>
    /// Represents the global data area in the script.
    /// </summary>
    internal class GlobalData : DataGroup
    {
    }
}
