namespace AGS.Editor
{
    public struct ScriptModuleDef
    {
        public string Name;
        public string Author;
        public string Description;
        public string Version;
        public string Script;
        public string Header;
        public int UniqueKey;
    }
}
