using System;

namespace AGS.Types
{
    // Problem is that every platform has their own, so we try to be as universal as possible here
    public enum RuntimeAudioDriver
    {
        Default = -1,
        Disabled = 0
    }
}
