using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace AGS.Types
{
    public interface IFloatWindow
    {
        Point Location { get; }
        Size Size { get; }
    }
}
