//
//  plugin_registration.hpp
//  Shivah
//
//  Created by Nick Sonneveld on 26/06/2016.
//
//

#ifndef plugin_registration_hpp
#define plugin_registration_hpp


#endif /* plugin_registration_hpp */
