using System.Reflection;
using System.Runtime.CompilerServices;
using System.Runtime.InteropServices;

[assembly: AssemblyTitle("AGS Script Compiler")]
[assembly: AssemblyDescription("AGS Script Compiler .NET")]
[assembly: AssemblyConfiguration("")]
[assembly: AssemblyCompany("Chris Jones")]
[assembly: AssemblyProduct("Adventure Game Studio")]
[assembly: AssemblyCopyright("Copyright © 2008 Chris Jones")]
[assembly: AssemblyTrademark("")]
[assembly: AssemblyCulture("")]
[assembly: AssemblyVersion(AGS.Types.Version.AGS_EDITOR_VERSION)]
[assembly: AssemblyFileVersion(AGS.Types.Version.AGS_EDITOR_VERSION)]
