using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types
{
    public enum RoomAreaMaskType
    {
        None,
        Hotspots,
        WalkBehinds,
        WalkableAreas,
        Regions
    }
}
