using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.CScript.Compiler
{
    public class Constants
    {
        public const string NEW_SCRIPT_MARKER = "\"__NEWSCRIPTSTART_";
    }
}
