using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types
{
    public enum RoomTransitionStyle
    {
        FadeOutAndIn = 0,
        Instant = 1,
        Dissolve = 2,
        BlackBoxOut = 3,
        CrossFade = 4
    }
}
