using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Editor
{
    public enum GUIAddType
    {
        None = 0,
        Button,
        Label,
        TextBox,
        ListBox,
        Slider,
        InvWindow
    }
}
