using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types
{
    public enum FontOutlineStyle
    {
        None = 0,
        Automatic = 1,
        UseOutlineFont = 2
    }
}
