//{{NO_DEPENDENCIES}}
// Microsoft Visual C++ generated include file.
// Used by version.rc
//
#define IDOKRUN                         3
#define IDRESETTODEFAULTS               4
#define IDI_ICON                        101
#define IDD_SETUP                       102
#define IDD_PAGE_BASIC                  109
#define IDD_PAGE_ADVANCED               110
#define IDD_PAGE_PATHS                  111
#define IDD_PAGE_ACCESS                 112
#define IDC_TABPANEL                    1000
#define IDC_DIGISOUND                   1003
#define IDC_WINDOWED                    1007
#define IDC_VERSION                     1008
#define IDC_LANGUAGE                    1009
#define IDC_SPRITECACHE                 1010
#define IDC_RENDERATSCREENRES           1011
#define IDC_DISPLAYINDEX                1012
#define IDC_FULLSCREENDESKTOP           1013
#define IDC_TEXTURECACHE                1014
#define IDC_SPRITECACHE3                1015
#define IDC_SOUNDCACHE                  1015
#define IDC_VOICEPACK                   1016
#define IDC_ANTIALIAS                   1021
#define IDC_GFXFILTER                   1023
#define IDC_GFXDRIVER                   1024
#define IDC_RESOLUTION                  1025
#define IDC_GFXMODE                     1028
#define IDC_VSYNC                       1031
#define IDC_FSSCALING                   1033
#define IDC_WINDOWSCALING               1034
#define IDC_GFXOPTIONS                  1035
#define IDC_MOUSESPEED                  1036
#define IDC_MOUSESPEED_TEXT             1037
#define IDC_MOUSE_AUTOLOCK              1038
#define IDC_CUSTOMSAVEDIRCHECK          1039
#define IDC_CUSTOMSAVEDIR               1040
#define IDC_CUSTOMSAVEDIRBTN            1041
#define IDC_CUSTOMAPPDATADIRCHECK       1045
#define IDC_CUSTOMAPPDATADIR            1046
#define IDC_CUSTOMAPPDATADIRBTN         1047
#define IDC_SPEECHSKIPSTYLE             1048
#define IDC_TEXTSKIPSTYLE               1049
#define IDC_ACCESSENABLECHECK           1050
#define IDC_LABEL_SPEECHSKIPSTYLE       1051
#define IDC_LABEL_TEXTSKIPSTYLE         1052
#define IDC_LABEL_TEXTREADSPEED         1053
#define IDC_TEXTREADSPEED               1054
#define IDC_TEXTREADSPEED_TEXT          1055

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        112
#define _APS_NEXT_COMMAND_VALUE         40001
#define _APS_NEXT_CONTROL_VALUE         1055
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
