using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Editor
{
    internal class ComponentIDs
    {
        public const string GeneralSettings = "GeneralSettings";
        public const string DefaultSetup = "DefaultSetup";
        public const string GUIs = "GUIs";
        public const string SourceControl = "SourceControl"; // [DEPRECATED]
        public const string Plugins = "Plugins";
        public const string Inventory = "Inventory";
        public const string Dialogs = "Dialogs";
        public const string Characters = "Characters";
        public const string Audio = "AudioNew";
        public const string Palette = "Palette";
        public const string TextParser = "TextParser";
        public const string GlobalVariables = "GlobalVariables";
        public const string Welcome = "Welcome";
        public const string LipSync = "LipSync";
        public const string SpriteManager = "SpriteManager";
        public const string ViewEditor = "Views";
        public const string Scripts = "Scripts";
        public const string FileCommands = "FileCommands";
        public const string BuildCommands = "BuildCommands";
        public const string HelpCommands = "HelpCommands";
        public const string Cursors = "Cursors";
        public const string Fonts = "Fonts";
        public const string Translations = "Translations";
        public const string Speech = "MusicAndSpeech";
        public const string Rooms = "Rooms";
        public const string DebugLog = "DebugLog";
        public const string StatisticsSender = "StatisticsSender";
        public const string CustomProperties = "CustomProperties";
    }
}
