using System;
using System.Collections.Generic;
using System.Text;

namespace AGS.Types
{
    public enum CustomPropertyAppliesTo
    {
        Characters,
        Hotspots,
        InventoryItems,
        Objects,
        Rooms
    }
}
